// a transient league stores a history buffer of filtered
// data to allow continuous filtering, searches filtered data
// for peaks, and maintains a list of those peaks sorted by
// amplitude. The league is of fixed length, ie, the number
// of peaks stored is predetermined.
//
// Edward Daw, 3rd July 2001

// C++ includes
#include <iostream>
#include <vector>
#include <string>
#include <stdio.h>

// root includes for float histogram class
// #include "TF1.h"

// dmt includes
#include "TSeries.hh"
#include "FSeries.hh"
#include "Dacc.hh"
#include "Interval.hh"
#include "Time.hh"
#include "DVector.hh"
#include "TSeries.hh"

#include "xsil/Xwriter.hh"
#include "xsil/xsil.hh"

// dmt includes
#include "TransientLeague.hh"
#include "Event.hh"

using namespace std;

TransientLeague::
TransientLeague(TSeries tsinit, int nkeep, int peak_width_bins,
		int event_store_width_bins, string monserver_name) {
  initialize(tsinit, nkeep, peak_width_bins,event_store_width_bins,
	     monserver_name);
  return;
}

TransientLeague::
TransientLeague(TSeries tsinit, TSeries tsraw, int nkeep, 
		int peak_width_bins, int event_store_width_bins, 
		string monserver_name ) {
  initialize(tsinit, tsraw, nkeep, peak_width_bins, event_store_width_bins,
	     monserver_name);
}

int
TransientLeague::
initialize(TSeries tsinit, TSeries tsraw, int nkeep, int peak_width_bins,
	   int event_store_width_bins, string monserver_name) {
  initialize(tsinit, nkeep, peak_width_bins, event_store_width_bins,
	     monserver_name);

#ifdef DEBUGSLOPE
  cout << "before test for raw timeseries empty" << endl;
#endif

  if( !(tsraw.isEmpty()) ) {
    // get raw data for times of initial event
    TSeries rawdisplay(tsraw.extract(_initial_event.getViewerStartTime(),
				     _initial_event.getViewerDuration()));

    // initialize timeseries memory for raw data for each event 
    // in the league

#ifdef DEBUGSLOPE
    cout << "raw data used to initialize has start time "
	 << tsraw.getStartTime() << " and end time "
	 << tsraw.getEndTime() << endl;
    cout << "initial raw event starts at "
	 << rawdisplay.getStartTime() << " and ends at "
	 << rawdisplay.getEndTime() << " with  "
	 << rawdisplay.getNSample() << " bins." << endl;
    cout << "initial filtered event starts at "
	 << (_initial_event.getDisplay()).getStartTime() 
	 << " and ends at " << (_initial_event.getDisplay()).getEndTime()
	 << " with " << (_initial_event.getDisplay()).getNSample() << " bins."
	 << endl;
    cout << "allocatimg memory to timeseries for storing raw displays"
	 << endl;
#endif

    // allocate memory to raw event displays

    _p_raw_event_displays = new TSeries* [ _max_number_of_peaks_kept ];
    for(int i=0; i<_max_number_of_peaks_kept ; ++i) {
      _p_raw_event_displays[i] = new TSeries (rawdisplay);
    }

    _current_raw_tseries = new TSeries (rawdisplay);
    _raw_timeseries_allocated = true;
    
    // initialize history buffer to hold raw timeseries data
    _raw_history_buffer = new TSeries 
      ( tsraw.extract(_history_buffer->getStartTime(), 
		      _history_buffer->getInterval() ));
    
#ifdef DEBUGSLOPE
    cout << "raw data history buffer starts at " 
	 << _raw_history_buffer->getStartTime() << " and ends at "
	 << _raw_history_buffer->getEndTime() << " with "
	 << _raw_history_buffer->getNSample() << " samples." << endl;
#endif

    // point raw display for back of event list and initial event
    // to the end of the array of allocated timeseries segments

    _initial_event.addRawDisplay( _p_raw_event_displays[ _push_counter ] );

    // add raw display pointer to the only event in the peak vector

    (peak_vector.back()).addRawDisplay
      ( _p_raw_event_displays[ _push_counter ]);

    // add raw display pointer to the initial event

    _initial_raw_event_tseries = new TSeries
      ( _initial_event.getRawDisplay() );

  } else {
    _raw_timeseries_allocated = false;
  }

#ifdef DEBUGSLOPE
  cout << "finished initializing transient league" << endl;
#endif

  return 0;
}


int
TransientLeague::
initialize(TSeries tsinit, int nkeep, int peak_width_bins,
		int event_store_width_bins, string monserver_name) {
  // if this function is called directly as opposed to from other
  // initialize function prototypes, no memory space for raw,
  // unfiltered data will be allocated
  _raw_timeseries_allocated = false;

  // set the name of the monitor server
  _monserver_name = monserver_name;

  // set counter for number of new pushes to event vector to zero
  _push_counter = 0;

  // set up histogram 
  _filthist = new Histogram1("all filtered cumulative",512,-1000,1000,
			     "filtered noise (amp / sqrt(bin))",
			     "dN/d(amp)");
  _periodfilthist = new Histogram1("all filtered cumulative",512,-1000,1000,
				   "filtered noise (amp / sqrt(bin))",
				   "dN/d(amp)");

  // set the maximum number of stored peaks
  _max_number_of_peaks_kept = nkeep;

  // set the number of bins on each side of the peak maximum that
  // you want to be stored in the timeslice
  _viewer_width = event_store_width_bins;

  // set the width of a peak in bins
  _peak_width = peak_width_bins;

  // make an internal copy of the timeseries passed to the league
  TSeries data_in(tsinit);

  // write start time for whole league
  _league_start_time = data_in.getStartTime();

  // convert the data in the internal copy to type double
  data_in.Convert(DVector::t_double);

  // copy the later end of tsinit to the history buffer
  // interval 0.0 means that all the data from history start
  // until the end of data_in is written to _history_buffer
  Time historystart( data_in.getEndTime() - 
		     2.0 * _viewer_width * data_in.getTStep() );
  _history_buffer = new TSeries ( data_in.extract( historystart,
						   Interval(0.0) ) );
  // set history allocated bit to true
  _history_allocated = true;

  // count the elements in the internal copy
  int ns = (int)data_in.getNSample();

  // if the length of the internal copy is less than three times
  // _viewer_width, then the skirts of the biggest bin stored
  // in the viewer time series might fall
  // off one or other end of the timeseries. Therefore exit if
  // the timeseries is too short
  if(ns < 3*_viewer_width) {
    _in_use = false;
    cerr << "ERROR: input time series is too short. Init failed." << endl;
    return -1;
  }

  // allocate memory for an array of doubles to hold the data
  double* double_data = new double [ns];
  
  // extract the data to an array of doubles
  data_in.getData((size_t)ns,double_data);

  // get a pointer to the biggest element in the middle section of
  // the array
  double* biggest_element = max_element(double_data+_viewer_width,
					double_data+ns-_viewer_width);

  // get the time of this element
  Time event_time = data_in.getBinT( (size_t)(biggest_element - double_data) );

  // get interval corresponding to _viewer_width and _peak_width
  Interval viewer_width_time = double(_viewer_width) * data_in.getTStep();
  // Interval peak_width_time = _peak_width * data_in.getTStep();

  // make a timeseries of the _peak_width bins before and after this
  // element
  TSeries event_timeslice = data_in.extract( event_time - viewer_width_time,
					     2.0*viewer_width_time );

  // allocate memory for storage of the timeseries of the initial event
  _initial_event_tseries = new TSeries (event_timeslice);

  // allocate memory for the array of pointers for biggest events and
  // set all of them to point initially to the timeseries for
  // the initial event
  _p_biggest_event_displays = new TSeries* [_max_number_of_peaks_kept];

  for(unsigned int i=0;i<(unsigned int)_max_number_of_peaks_kept;++i) {
    // assign initial event timeseries to all event storage elements
    _p_biggest_event_displays[i] = new TSeries (*(_initial_event_tseries));
  }

  // write the element to the initial event
  Event stack_event( _p_biggest_event_displays[0], event_time,
		    double(_peak_width)*data_in.getTStep(), *biggest_element );
  Event initial_event( _initial_event_tseries, event_time,
		      double(_peak_width)*data_in.getTStep(), *biggest_element );
  _initial_event = initial_event;

#ifdef DEBUGSLOPE
  cout << "adding initial event" << endl;
#endif
  addEvent(stack_event);

  // free memory allocated for array of doubles
  delete [] double_data;

  _in_use = true;

  _history_append_fail = false;

  return 0;
}

TransientLeague::TransientLeague(void) {
  _in_use = false;
  _history_allocated = false;
  return;
}

// return pointer to event array at position i 
// ( 0 <= i < _max_number_of_peaks_kept )
TSeries*
TransientLeague::monitorUpdate(int i) {
  return _p_biggest_event_displays[i];
}

// return pointer to raw event array at position i
TSeries*
TransientLeague::monitorRawUpdate(int i) {
  return _p_raw_event_displays[i];
}

void 
TransientLeague::addEvent(Event e) {

  // copy the event timeseries data into the static timeseries array
#ifdef DEBUGSLOPE
  cout << "number of events is " << getNEvents() << endl;
#endif

  int lastindex = _max_number_of_peaks_kept - 1;

  TSeries* pts;
  TSeries* prts;

  if( getNEvents() < lastindex ) {
    pts = _p_biggest_event_displays[ getNEvents() ];
    if(_raw_timeseries_allocated) {
      prts = _p_raw_event_displays[ getNEvents() ];
#ifdef DEBUGSLOPE
      cout << "raw time series is at " << prts << endl;
      cout << "size of raw time series is " << sizeof(*prts) << endl;
#endif
    }
  } 

  else {
   
    pts = (peak_vector.back()).getDisplayPointer();
    if(_raw_timeseries_allocated) {
      prts = (peak_vector.back()).getRawDisplayPointer();

#ifdef DEBUGSLOPE
      cout << "raw time series is at " << prts << endl;
#endif

    }
    peak_vector.pop_back();
  }

  if(!_raw_timeseries_allocated) {

    // write timeseries data for event e
    *(pts) = e.getDisplay();

    Event newstackevent( pts,
			 e.getTime(),
			 e.getDuration(),
			 e.getAmplitude() );

    peak_vector.push_back(newstackevent);

  } 

  else {

#ifdef DEBUGSLOPE
    cout << "adding event with raw timeseries" << endl;
    cout << "size of event to be added is " << sizeof(e.getRawDisplay())
	 << endl;
#endif

    // write timeseries data for event e

    *(pts) = e.getDisplay();
    *(prts) = e.getRawDisplay();

    Event newstackevent( pts,
			 prts,
			 e.getTime(),
			 e.getDuration(),
			 e.getAmplitude() );

    peak_vector.push_back(newstackevent);

#ifdef DEBUGSLOPE
      cout << "added event with raw timeseries" << endl;
#endif

  }
  return;
}

int
TransientLeague::dumpAmplitudes(void) {
  for(vector<Event>::iterator i=peak_vector.begin();
      i<peak_vector.end() ; ++i) {
    Interval time_offset = i->getTime() - _league_start_time;
    cout << "at " << time_offset << " secs, amplitude " 
	 << i->getAmplitude() << " counts" << endl;
  }
  return 0;
}
  
int
TransientLeague::getHistoryLength(void) {
  return 2 * _viewer_width;
}

int 
TransientLeague::
search(TSeries ts, TSeries rts, int bins_to_skip, bool cont_data_flag) {
  if ( !_raw_timeseries_allocated ) {
    cout << "no facility for keeping raw timeseries data" << endl;
    return -1;
  } else {
    (*(_current_raw_tseries)) = rts;
    search(ts, bins_to_skip, cont_data_flag);
    return 0;
  }
}

int 
TransientLeague::
search(TSeries ts, int bins_to_skip, bool cont_data_flag) {

  // check that history buffer has been initialized
  if( !_history_allocated ) {
    throw invalid_argument("History Not Allocated");
  }

  // only check for contiguous data if cont_data_flag was not already false
  if( cont_data_flag ) {
    // check that new data contiguous with end of history buffer
    if( ts.getStartTime() != _history_buffer->getEndTime() ) {
      throw invalid_argument("Noncontiguous Data");
    }
  }

  // extract length of timeseries
  int new_data_length = (int)ts.getNSample();

  if(new_data_length - bins_to_skip < _peak_width) {
    throw invalid_argument("Data length shorter than peak width");
  }

  // copy input timeseries to internal stack
  TSeries newdata(ts);

  // convert timeseries to type double
  newdata.Convert(DVector::t_double);

  // make new time series consisting of the input series
  // with the history buffer added to the beginning
  TSeries rawdata;
  TSeries data;
  if( _history_append_fail == false) {
    data = (*_history_buffer);
    int appenderror = data.Append(newdata,1);
    if( appenderror != 0 ) {
      cout << "filtered data append failed with error " 
	   << appenderror << endl;
      _history_append_fail = true;
    }
    // this step only taken if raw data is also allocated
    if( _raw_timeseries_allocated ) {
      TSeries rd( *(_raw_history_buffer) );
      int rawappenderror = rd.Append((*_current_raw_tseries),1);
      if( rawappenderror != 0 ) {
	cout << "data append to raw data failed with error" 
	     << rawappenderror << endl;
      }
      rawdata = rd;
    }
    if( _history_append_fail == true ) {
      return -1;
    }
  } else {
    data = newdata;
    // this step only taken if raw data is also allocated
    if( _raw_timeseries_allocated ) {
      TSeries rd(*_current_raw_tseries);
      rawdata = rd;
    }
    _history_append_fail = false;
  }

  // if raw data is being handled in parallel, make a timeseries
  // here out of the existing history buffer and the new raw data

#ifdef DEBUGSLOPE
  cout << "raw timeseries is of length " << rawdata.getNSample() << endl;
  cout << "history runs from " 
       << _raw_history_buffer->getStartTime() << " to "
       << _raw_history_buffer->getEndTime() << endl;
  cout << "current raw timeseries runs from " 
       << _current_raw_tseries->getStartTime() << " to "
       << _current_raw_tseries->getEndTime() << endl;
  cout << "overall raw data runs from " 
       << rawdata.getStartTime() << " to " << rawdata.getEndTime()
       << endl;
  cout << "raw data contains " << rawdata.getNSample()
       << " and sampling time " << rawdata.getTStep() << endl;
#endif

  // extract the data from the concatenated history buffer and new data
  // to an array of doubles
  double* double_data = new double [data.getNSample()];
  data.getData((size_t)data.getNSample(),double_data);

  // write the filtered data to the filtered data histogram
  addHistData( ALL_HOURLY_FILTERED , double_data, data.getNSample());

  // set a counter to the first bin you want to probe for peaks
  int data_counter = _viewer_width + bins_to_skip;
  
  // define an iterator for vectors of Events
  Event lowest_amplitude_event;
  
  // loop until the data counter reaches the end of the data set
  // to be examined

  while( data_counter < _viewer_width + new_data_length ) {
    
    // set the iterator to point to the beginning of peak_vector
    lowest_amplitude_event = peak_vector.back();

    // get the amplitude of the peak at the beginning of the vector
    // using the getAmplitude public member of the Event class. This
    // is the amplitude of the smallest event.
    double threshold = lowest_amplitude_event.getAmplitude();

    // if bin height exceeds lowest event amplitude...

    if( (double_data[data_counter] > threshold) || 
	((int)peak_vector.size() < _max_number_of_peaks_kept) ) {

      // locate the largest bin of the _peak_width bins starting
      // with the one that passed threshold
      double* biggest_in_peak = max_element(double_data + data_counter,
					    double_data + data_counter
					    + _peak_width);
      
#ifdef DEBUGSLOPE
      cout << "data counter at start of segment = "
	   << data_counter << " , biggest peak at "
	   << (int)(biggest_in_peak - double_data) << endl;
#endif      

      // get the time associated with this bin
      Time event_time = data.getBinT( int(biggest_in_peak - double_data) );

      // extract a timeseries about this event
      TSeries event_view( data.extract
			  ( event_time - double(_viewer_width) * data.getTStep(),
			    2.0 * _viewer_width * data.getTStep() ));
      
      TSeries* pevent_view = new TSeries(event_view);

      // if raw data is being handled, extract raw data corresponding
      // to the times of the event view in the filtered data
      TSeries* praw_event_view=0;
      if( _raw_timeseries_allocated ) {

	//	TSeries revw( rawdata.extract( event_view.getStartTime(),
	//		       event_view.getInterval() ) );

	TSeries revw( rawdata.extract
		      ( event_time - double(_viewer_width) * data.getTStep(),
			2.0 * _viewer_width * data.getTStep() ));

	praw_event_view = new TSeries (revw);

      }

      if(_raw_timeseries_allocated) {
#ifdef DEBUGSLOPE
	cout << "creating new event with raw data" << endl;
#endif      
	Event new_peak( pevent_view, praw_event_view,
			event_time, double(_peak_width)*data.getTStep(),
			*biggest_in_peak );
#ifdef DEBUGSLOPE
	cout << "adding event to stack" << endl;
#endif
	addEvent(new_peak);
      } else {
	Event new_peak( pevent_view, event_time, 
			double(_peak_width)*data.getTStep(),
			*biggest_in_peak );
	addEvent(new_peak);
      }

      // sort the events by event size
#ifdef DEBUGSLOPE
      cout << "sorting events by amplitude" << endl;
      for ( vector<Event>::iterator i = peak_vector.begin();
	    i < peak_vector.end() ; ++i ) {
	i->shortTextSummary(cout);
      }
#endif
      sort(peak_vector.begin() , peak_vector.end() , greater<Event>() );
#ifdef DEBUGSLOPE
      cout << "after sorting:" << endl;
      for ( vector<Event>::iterator i = peak_vector.begin();
	    i < peak_vector.end() ; ++i ) {
	i->shortTextSummary(cout);
      }
#endif
		      
      // rearrange the static array of pointers to timeseries
      // to reflect the new event ordering

#ifdef DEBUGSLOPE
      cout << "re-sorting timeseries pointers" << endl;
#endif

      for( int i=0; i < getNEvents() ; ++i ) {
	_p_biggest_event_displays[i] = 
	  (peak_vector.begin() + i)->getDisplayPointer();
	if( _raw_timeseries_allocated ) {
	  _p_raw_event_displays[i] =
	    (peak_vector.begin() + i)->getRawDisplayPointer();
	}
      }

      // jump past bins included in event
      data_counter += _peak_width;

      // delete allocated memory
      if( _raw_timeseries_allocated ) {
	delete praw_event_view;
      }
      delete pevent_view;
    }
    
    // otherwise just go on to next bin
    else {
      ++data_counter;
    }
  }
  delete [] double_data;

  // update history buffer
  Interval history_length(2.0 * _viewer_width * data.getTStep());
  *(_history_buffer) = data.extract( data.getEndTime() - history_length,
				  history_length );

  // if raw data is being kept, update history buffer for raw data too
  if(_raw_timeseries_allocated ) {
    *(_raw_history_buffer) = rawdata.extract
      ( data.getEndTime() - history_length, history_length );
  }

  // return
  return 0;
}

int
TransientLeague::dumpLeague(void) {
  outputLeague(cout,true);
  return 0;
}

int
TransientLeague::outputHistogramContents(std::ostream& os, Histogram1* hist) {
  // write histogram bin contents
  os << "# start of histogram data" << endl;
  double* histdata = new double [ hist->GetNBins() + 2 ];
  hist->GetBinContents(histdata);
  os << "# underflow:" << endl;
  os << histdata[0] << endl;
  os << "# overflow:" << endl;
  os << histdata[hist->GetNBins() + 1] << endl;
  os << "# lower edge" << endl;
  os << hist->GetBinLowEdge(1) << endl;
  os << "# upper edge" << endl;
  os << hist->GetBinLowEdge( hist->GetNBins() + 1 ) << endl;
  os << "# number of bins of equal width:" << endl;
  os << hist->GetNBins() << endl;
  os << "# number of data elements: " << endl;
  os << hist->GetNEntries() << endl;
  os << "# bin contents start here" << endl;
  for(int i=1; i< (hist->GetNBins()+1); ++i) {
    os << histdata[i] << ", ";
  }
  os << endl;
  os << "# bin contents end here" << endl;
  delete [] histdata;
  cout << "wrote a histogram with " << hist->GetNEntries() << " entries" << endl;
  return 0;
}

int 
TransientLeague::outputLeague(std::ostream& os, bool eseg_write) {
  for(vector<Event>::iterator i = peak_vector.begin();
      i < peak_vector.end(); ++i) {
    if(eseg_write) {
      i->textSummary(os);
    } else {
      i->shortTextSummary(os);
    }
  }
  os << "# period histogram follows:" << endl;
  outputHistogramContents(os,_periodfilthist);
  os << "# cumulative histogram follows:" << endl;
  // make and print out cumulative histogram
  Histogram1* sumhist = new Histogram1((*(_filthist)) + (*(_periodfilthist)));
  outputHistogramContents(os,sumhist);
  delete sumhist;
  return 0;
}  

TSeries*
TransientLeague::getEventHistory(int event_index) {
  vector <Event>::iterator event_it;
  if(event_index < 0) {
    event_it = peak_vector.begin();
  } else if (event_index >= getNEvents()) {
    event_it = peak_vector.end();
  } else {
    event_it = peak_vector.begin() + event_index;
  }
  return (event_it->getDisplayPointer());
}

int 
TransientLeague::getNEvents(void) {
  vector<Event>::iterator first = peak_vector.begin();
  vector<Event>::iterator last = peak_vector.end();
  return (int)(last - first);
}

Time
TransientLeague::getEventTime(int event_index) {
  vector<Event>::iterator ev = peak_vector.begin() + event_index;
  return ev->getTime();
}

Interval
TransientLeague::getEventDuration(int event_index) {
  vector<Event>::iterator ev = peak_vector.begin() + event_index;
  return ev->getDuration();
}

Time
TransientLeague::startTime(void) {
  return _league_start_time;
}

bool
TransientLeague::inUse(void) {
  return _in_use;
}

bool
TransientLeague::rawDataStored(void) {
  return _raw_timeseries_allocated;
}

void
TransientLeague::RestartEventList(Time start_time) {
  if(_in_use) {

    // construct a new event with the data from the smallest event
    // already in the league

#ifdef DEBUGSLOPE
    cout << "constructing timeseries for new initial event" << endl;
#endif

    TSeries oldtseries( (peak_vector.back()).getDisplay() );
    Time oldtime( (peak_vector.back()).getTime() );
    Interval oldduration( (peak_vector.back()).getDuration() );
    double oldamplitude( (peak_vector.back()).getAmplitude() );
    TSeries oldrawtseries;
    
    if( _raw_timeseries_allocated ) {
      TSeries oldrawts( (peak_vector.back()).getRawDisplay() );
      oldrawtseries = oldrawts;
    }

    // reset the timeseries data and event data for the initial event
    // field in the timeseries to the values for the smallest event in
    // the league

#ifdef DEBUGSLOPE
    cout << "setting contents of initial event timeseries" << endl;
#endif

    *(_initial_event_tseries) = oldtseries;
    if( _raw_timeseries_allocated ) {
      *(_initial_raw_event_tseries) = oldrawtseries;
    }

#ifdef DEBUGSLOPE
    cout << "creating event for new initial event" << endl;
#endif

    Event smallestold;
    if( _raw_timeseries_allocated ) {
      Event so( _initial_event_tseries, _initial_raw_event_tseries,
		oldtime,oldduration,oldamplitude );
      smallestold = so;
    } else {
      Event so(_initial_event_tseries, oldtime, oldduration, oldamplitude);
      smallestold = so;
    }

#ifdef DEBUGSLOPE
    cout << "setting initial event" << endl;
#endif
    _initial_event = smallestold;

    // reset all the timeseries in the static event storage array to the
    // new initial event
#ifdef DEBUGSLOPE
    cout << "resetting contents of event time series" << endl;
#endif
    for( int i=0; i<_max_number_of_peaks_kept ; ++i ) {
#ifdef DEBUGSLOPE
      cout << "setting event " << i << endl;
#endif
      *(_p_biggest_event_displays[i]) = oldtseries;
    }
    if(_raw_timeseries_allocated) {
      for( int i=0; i<_max_number_of_peaks_kept ; ++i ) {
	*(_p_raw_event_displays[i]) = oldrawtseries;
      }
    }

    // add the smallest event to the vector of events.

#ifdef DEBUGSLOPE
    cout << "resetting contents of peak vector" << endl;
#endif
    peak_vector.erase(peak_vector.begin(), peak_vector.end() );

#ifdef DEBUGSLOPE
    cout << "adding smallest event to beginnning of event vector" << endl;
#endif
    addEvent(smallestold);

#ifdef DEBUGSLOPE
    cout << "setting league start time" << endl;
#endif
    _league_start_time = start_time;

#ifdef DEBUGSLOPE
    cout << "resetting period histogram contents" << endl;
#endif
    (*(_filthist)) += (*(_periodfilthist));
    _periodfilthist->Clear();

  }
  return;
}

int
TransientLeague::addHistData(unsigned int histident, double* data, int ndata) {
  switch(histident) {
  case ALL_HOURLY_FILTERED:
    {
      for(int i=0;i<ndata;++i) {
	_periodfilthist->Fill(data[i]);
      }
      cout << "Total of " << _periodfilthist->GetNEntries() 
	   << " entries." << endl;
      break;
    }
  case ALL_CUMULATIVE_FILTERED: 
    {
      for(int i=0;i<ndata;++i) {
	_filthist->Fill(data[i]);
      }
      cout << "Total of " << _filthist->GetNEntries() 
	   << " entries." << endl;
      break;
    }
  default:
    cout << "unsupported histogram" << endl;
    return -1;
  }
  return 0;
}

Histogram1*
TransientLeague::getHistPointer(unsigned int histident) {
  switch(histident) {
  case ALL_HOURLY_FILTERED:
    {
      return _periodfilthist;
      break;
    }
  case ALL_CUMULATIVE_FILTERED:
    {
      return _filthist;
      break;
    }
  default:
    cout << "unsupported histogram" << endl;
    return NULL;
  }
  return 0;
}

TransientLeague::~TransientLeague( void ) {
  if( _in_use ) {

    // clear vector of events
    peak_vector.erase(peak_vector.begin() , peak_vector.end() );

    // clear memory allocated to filtered data timeseries
    for(int i=0;i<_max_number_of_peaks_kept;++i) {
      delete (_p_biggest_event_displays[i]);
    }
    delete [] _p_biggest_event_displays;
    delete _initial_event_tseries;

    // clear histogram storage
    delete _filthist;
    delete _periodfilthist;

    // clear memory allocated to raw unfiltered data timeseries
    if( _raw_timeseries_allocated ) {
      for(int i=0;i<_max_number_of_peaks_kept; ++i) {
	delete (_p_raw_event_displays[i]);
      }
      delete [] _p_raw_event_displays;
      delete _current_raw_tseries;
      delete _raw_history_buffer;
      delete _initial_raw_event_tseries;
    }
  }

  // clear memory allocated to history buffer
  if( _history_allocated ) {
    delete _history_buffer;
  }
}











